{
********************************************************************************
Modolo:        SxTable
Versione:      1.1
Descrizione    sort table e temp table
Autore:        Michele Palmieri
E-mail         Palmieri@ItaliaBusiness.Net
1.0            26/03/1999
********************************************************************************
}

unit SxTable;

interface

uses
  SysUtils, Windows, DBTables, Classes,DBITypes, Db;

type

{ TTmpTable }

  TSxTable = class(TTable)
  private
    hCursor: hDBICur;
    FTemporal,
    FPermanent: Boolean;
    FTmpFile,
    FTmpDir:String;
    OpenSortedTable: boolean;
  protected
    tmpFile:String;
    procedure OpenCursor(InfoQuery: Boolean); override;
    procedure CloseCursor; override;
    function GetDBHandle: HDBIDB;
    procedure SetPermanent(Value:Boolean);
    procedure SetTemporal(Value:Boolean);
  public
    destructor Destroy; override;
    constructor Create(AOwner: TComponent); override;
    procedure CreateTmpTable;
    procedure Sort(Field: TField);
    procedure SortEx(Field: TField; Order: SORTOrder;
                 CaseInsensitive: BOOL; var NumberToSort: longint);
    function  GetUniqueFileName: string;
    function  GetTempDirectory: string;
  published
    property Permanent: Boolean read FPermanent write SetPermanent;
    property Temporal: Boolean read FTemporal write SetTemporal;
  end;


procedure Register;

implementation

function  TSxTable.GetUniqueFileName: string;
var
    a:Pchar;
    S:string;
begin
   a:=stralloc(255);
   s:=GetTempDirectory + #0;
   {$ifdef win32}
      getTempfilename(@s[1],'DB'+#0,0,a);
   {$else}
      getTempfilename(#0,'DB'+#0,1,a);
   {$endif}
  s:=changefileext(extractfilename(StrPas(a)),'.db');
  tmpFile:=StrPas(a);
  Result := s;
  strdispose(a);
//  Result := Format('%8s', [DateTimeTostrf(time())]);
end;

function  TSxTable.GetTempDirectory: string;
begin
  SetLength(Result, 255);
  GetTempPath(255, PChar(Result));
end;

procedure TSxTable.OpenCursor(InfoQuery: Boolean);
Begin
  if FTemporal then
  Begin
     try
       if FTmpfile = '' then
          createTmpTable;
     except
     raise;
       exit;
     end;
  end;
  inherited OpenCursor(InfoQuery);
End;

constructor TSxTable.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTemporal := False;
end;

procedure TSxTable.CreateTmpTable;
begin
  if active then exit;
  FTemporal := True;
  try
    DatabaseName := GetTempDirectory;
    TableName    := GetUniqueFileName;
    FTmpFile:=TableName;
    FTmpDir :=DatabaseName;
    CreateTable;
  except
    raise;
  end;
end;

procedure TSxTable.CloseCursor;
Var d,t:String;
begin
  inherited CloseCursor;
  if FTemporal  and not Permanent and (FTmpFile <>'') then
  Begin
      t:=TableName;
      d :=DatabaseName;
      TableName:=FTmpFile;
      DatabaseName:=FTmpDir;
      try
        DeleteTable;
        SysUtils.deletefile(TmpFile);
      except
        raise;
      end;
      TableName:=t;
      DatabaseName:=d;
      FTmpFile:='';
      FTmpDir:='';
  end
end;

destructor TsxTable.Destroy;
begin
  if Fpermanent then
   if (FTmpFile <>'') then
   try
       DeleteTable;
   except
   raise;
   end;
  inherited Destroy;
end;

function TsxTable.GetDBHandle: HDBIDB;
Var hDb:HDBIDB;
    s:string;
begin
  if Database <> nil then
    Result := Database.Handle else
  begin
    if FTemporal then
    Begin
        Check(DbiOpenDatabase(nil, nil, dbiREADONLY, dbiOPENSHARED,
                nil, 0, nil, nil, hDb));
        s:=  DatabaseName;
        Check(DbiSetDirectory(hDb, PChar(s)));
    end
    else
    Begin
        s:=  DatabaseName;
        Check(DbiOpenDatabase(PChar(s), nil, dbiReadWrite, dbiOpenShared,
          nil, 0, nil, nil, hdb));
    end;

    Result := hDb;
  end;
end;

procedure TsxTable.Sort(Field: TField);
var
  SortNumber: longint;
  FieldNumber: word;
  CaseInsensitive: BOOL;
  Order: SORTOrder;
  FTableName:String;
  apermanent:Boolean;
begin
  CaseInsensitive := TRUE;
  FieldNumber := Field.index + 1;
  Check(DbiGetRecordCount(Handle, SortNumber));
  Order := sortASCEND;
  FTableName:=TableName;
  apermanent := permanent;
  permanent:= True;
  OpenSortedTable := True;
  Active := False;
  try
    Check(DbiSortTable(GetDBHandle , PChar(FTableName), PChar(TableType),
               nil, nil, nil, nil, 1, @FieldNumber, @CaseInsensitive, @Order, nil,
               False, nil, SortNumber));
    Active := True;
  finally
    permanent := apermanent;
    OpenSortedTable := False;
  end;
end;

procedure TsxTable.SortEx(Field: TField; Order: SORTOrder;
     CaseInsensitive: BOOL; var NumberToSort: longint);
var
  FieldNumber: word;
  FTableName:String;
  apermanent:Boolean;
begin
  FieldNumber := Field.index + 1;
  FTableName:=TableName;
  apermanent := permanent;
  permanent:= True;

  OpenSortedTable := True;
  Active := False;
  try
    Check(DbiSortTable(GetDBHandle, PChar(FTableName), PChar(TableType),
               nil, nil, nil, nil, 1, @FieldNumber, @CaseInsensitive,
               @Order, nil, False, nil, NumberToSort));
   Active := True;
  finally
    OpenSortedTable := False;
    permanent := apermanent;
  end;
end;

procedure TSxTable.SetPermanent(Value:Boolean);
Var d,t:String;
Begin
  if value = FPermanent then exit;
  FPermanent:=Value;
  if active then exit;
  if FTemporal  and not Permanent and (FTmpFile <>'') then
  Begin
      t:=TableName;
      d :=DatabaseName;
      TableName:=FTmpFile;
      DatabaseName:=FTmpDir;
      DeleteTable;
      TableName:=t;
      DatabaseName:=d;
      FTmpFile:='';
      FTmpDir:='';
  end
End;

procedure TSxTable.SetTemporal(Value:Boolean);
Var d,t:String;
Begin
  if value = FTemporal then exit;
  FTemporal:=Value;
  if active then exit;
  if FTemporal  and not Permanent and (FTmpFile <>'') then
  Begin
      t:=TableName;
      d :=DatabaseName;
      TableName:=FTmpFile;
      DatabaseName:=FTmpDir;
      DeleteTable;
      TableName:=t;
      DatabaseName:=d;
      FTmpFile:='';
      FTmpDir:='';
  end
End;

procedure Register;
begin
  RegisterComponents('Sx', [TSxTable]);
end;

end.

